import registryConstants from '@socketsecurity/registry/lib/constants';
import type { Agent } from './utils/package-environment.mts';
import type { Remap } from '@socketsecurity/registry/lib/objects';
type RegistryEnv = typeof registryConstants.ENV;
type RegistryInternals = (typeof registryConstants)['Symbol(kInternalsSymbol)'];
type Sentry = any;
type Internals = Remap<Omit<RegistryInternals, 'getIpc'> & Readonly<{
    getIpc: {
        (): Promise<IPC>;
        <K extends keyof IPC | undefined>(key?: K | undefined): Promise<K extends keyof IPC ? IPC[K] : IPC>;
    };
    getSentry: () => Sentry;
    setSentry(Sentry: Sentry): boolean;
}>>;
type ENV = Remap<RegistryEnv & Readonly<{
    DISABLE_GITHUB_CACHE: boolean;
    GITHUB_ACTIONS: boolean;
    GITHUB_REF_NAME: string;
    GITHUB_REF_TYPE: string;
    GITHUB_REPOSITORY: string;
    GITHUB_TOKEN: string;
    INLINED_CYCLONEDX_CDXGEN_VERSION: string;
    INLINED_SOCKET_CLI_HOMEPAGE: string;
    INLINED_SOCKET_CLI_LEGACY_BUILD: string;
    INLINED_SOCKET_CLI_NAME: string;
    INLINED_SOCKET_CLI_PUBLISHED_BUILD: string;
    INLINED_SOCKET_CLI_SENTRY_BUILD: string;
    INLINED_SOCKET_CLI_VERSION: string;
    INLINED_SOCKET_CLI_VERSION_HASH: string;
    INLINED_SYNP_VERSION: string;
    LOCALAPPDATA: string;
    NODE_COMPILE_CACHE: string;
    PATH: string;
    SOCKET_CLI_ACCEPT_RISKS: boolean;
    SOCKET_CLI_CONFIG: string;
    SOCKET_CLI_DEBUG: boolean;
    SOCKET_CLI_NO_API_TOKEN: boolean;
    SOCKET_CLI_VIEW_ALL_RISKS: boolean;
    SOCKET_SECURITY_API_BASE_URL: string;
    SOCKET_SECURITY_API_PROXY: string;
    SOCKET_SECURITY_API_TOKEN: string;
    SOCKET_SECURITY_GITHUB_PAT: string;
    TERM: string;
    XDG_DATA_HOME: string;
}>>;
type IPC = Readonly<{
    SOCKET_CLI_FIX?: string | undefined;
    SOCKET_CLI_OPTIMIZE?: boolean | undefined;
    SOCKET_CLI_SAFE_BIN?: string | undefined;
    SOCKET_CLI_SAFE_PROGRESS?: boolean | undefined;
}>;
type Constants = Remap<Omit<typeof registryConstants, 'Symbol(kInternalsSymbol)' | 'ENV' | 'IPC'> & {
    readonly 'Symbol(kInternalsSymbol)': Internals;
    readonly ALERT_TYPE_CRITICAL_CVE: 'criticalCVE';
    readonly ALERT_TYPE_CVE: 'cve';
    readonly ALERT_TYPE_MEDIUM_CVE: 'mediumCVE';
    readonly ALERT_TYPE_MILD_CVE: 'mildCVE';
    readonly API_V0_URL: 'https://api.socket.dev/v0/';
    readonly BINARY_LOCK_EXT: '.lockb';
    readonly BUN: 'bun';
    readonly CLI: 'cli';
    readonly CVE_ALERT_PROPS_FIRST_PATCHED_VERSION_IDENTIFIER: 'firstPatchedVersionIdentifier';
    readonly ENV: ENV;
    readonly DRY_RUN_LABEL: '[DryRun]';
    readonly DRY_RUN_BAILING_NOW: '[DryRun] Bailing now';
    readonly DRY_RUN_NOT_SAVING: '[DryRun] Not saving';
    readonly IPC: IPC;
    readonly LOCK_EXT: '.lock';
    readonly NPM_BUGGY_OVERRIDES_PATCHED_VERSION: '11.2.0';
    readonly NPM_REGISTRY_URL: 'https://registry.npmjs.org';
    readonly PNPM: 'pnpm';
    readonly REDACTED: '<redacted>';
    readonly SHADOW_BIN: 'shadow-bin';
    readonly SHADOW_INJECT: 'shadow-inject';
    readonly SOCKET: 'socket';
    readonly SOCKET_CLI_ACCEPT_RISKS: 'SOCKET_CLI_ACCEPT_RISKS';
    readonly SOCKET_CLI_BIN_NAME: 'socket';
    readonly SOCKET_CLI_BIN_NAME_ALIAS: 'cli';
    readonly SOCKET_CLI_CONFIG: 'SOCKET_CLI_CONFIG';
    readonly SOCKET_CLI_FIX: 'SOCKET_CLI_FIX';
    readonly SOCKET_CLI_ISSUES_URL: 'https://github.com/SocketDev/socket-cli/issues';
    readonly SOCKET_CLI_SENTRY_BIN_NAME_ALIAS: 'cli-with-sentry';
    readonly SOCKET_CLI_LEGACY_PACKAGE_NAME: '@socketsecurity/cli';
    readonly SOCKET_CLI_NPM_BIN_NAME: 'socket-npm';
    readonly SOCKET_CLI_NPX_BIN_NAME: 'socket-npx';
    readonly SOCKET_CLI_OPTIMIZE: 'SOCKET_CLI_OPTIMIZE';
    readonly SOCKET_CLI_PACKAGE_NAME: 'socket';
    readonly SOCKET_CLI_SAFE_BIN: 'SOCKET_CLI_SAFE_BIN';
    readonly SOCKET_CLI_SAFE_PROGRESS: 'SOCKET_CLI_SAFE_PROGRESS';
    readonly SOCKET_CLI_SENTRY_BIN_NAME: 'socket-with-sentry';
    readonly SOCKET_CLI_SENTRY_NPM_BIN_NAME: 'socket-npm-with-sentry';
    readonly SOCKET_CLI_SENTRY_NPX_BIN_NAME: 'socket-npx-with-sentry';
    readonly SOCKET_CLI_SENTRY_PACKAGE_NAME: '@socketsecurity/cli-with-sentry';
    readonly SOCKET_CLI_VIEW_ALL_RISKS: 'SOCKET_CLI_VIEW_ALL_RISKS';
    readonly SOCKET_WEBSITE_URL: 'https://socket.dev';
    readonly VLT: 'vlt';
    readonly WITH_SENTRY: 'with-sentry';
    readonly YARN: 'yarn';
    readonly YARN_BERRY: 'yarn/berry';
    readonly YARN_CLASSIC: 'yarn/classic';
    readonly YARN_LOCK: 'yarn.lock';
    readonly bashRcPath: string;
    readonly blessedOptions: {
        smartCSR: boolean;
        term: string;
        useBCE: boolean;
    };
    readonly distCliPath: string;
    readonly distInstrumentWithSentryPath: string;
    readonly distShadowBinPath: string;
    readonly distShadowInjectPath: string;
    readonly githubCachePath: string;
    readonly homePath: string;
    readonly minimumVersionByAgent: Map<Agent, string>;
    readonly nmBinPath: string;
    readonly nodeHardenFlags: string[];
    readonly rootBinPath: string;
    readonly distPath: string;
    readonly rootPath: string;
    readonly shadowBinPath: string;
    readonly socketAppDataPath: string;
    readonly socketCachePath: string;
    readonly zshRcPath: string;
}>;
declare const constants: Constants;
export default constants;
//# sourceMappingURL=constants.d.mts.map