import { Logger } from '@socketsecurity/registry/lib/logger';
import type { Remap } from '@socketsecurity/registry/lib/objects';
import type { EditablePackageJson } from '@socketsecurity/registry/lib/packages';
import type { SemVer } from 'semver';
export declare const AGENTS: readonly ["bun", "npm", "pnpm", "yarn/berry", "yarn/classic", "vlt"];
export type Agent = (typeof AGENTS)[number];
type EnvBase = {
    agent: Agent;
    agentExecPath: string;
    agentSupported: boolean;
    features: {
        npmBuggyOverrides: boolean;
    };
    nodeSupported: boolean;
    nodeVersion: SemVer;
    npmExecPath: string;
    pkgRequirements: {
        agent: string;
        node: string;
    };
    pkgSupports: {
        agent: boolean;
        node: boolean;
    };
};
export type EnvDetails = Readonly<Remap<EnvBase & {
    agentVersion: SemVer;
    editablePkgJson: EditablePackageJson;
    lockName: string;
    lockPath: string;
    lockSrc: string;
    pkgPath: string;
}>>;
export type PartialEnvDetails = Readonly<Remap<EnvBase & {
    agentVersion: SemVer | undefined;
    editablePkgJson: EditablePackageJson | undefined;
    lockName: string | undefined;
    lockPath: string | undefined;
    lockSrc: string | undefined;
    pkgPath: string | undefined;
}>>;
export type DetectOptions = {
    cwd?: string | undefined;
    onUnknown?: (pkgManager: string | undefined) => void;
};
export declare function detectPackageEnvironment({ cwd, onUnknown, }?: DetectOptions): Promise<EnvDetails | PartialEnvDetails>;
export type DetectAndValidateOptions = {
    cmdName?: string | undefined;
    logger?: Logger | undefined;
    prod?: boolean | undefined;
};
export declare function detectAndValidatePackageEnvironment(cwd: string, options?: DetectAndValidateOptions | undefined): Promise<void | EnvDetails>;
export {};
//# sourceMappingURL=package-environment.d.mts.map